from django.db import models


class ChartType(models.Model):

    """Describes possible chart types"""

    name = models.CharField(
        max_length=255, blank=True, help_text="A name of the chart type."
    )
    icon = models.CharField(
        max_length=255, blank=True,
        help_text="A HTML snippet which could be used to display a link to the chart"
    )

    def __str__(self):
        return self.name


class ChartConfig(models.Model):
    """A class to store config-info for Charts"""

    model_name = models.CharField(
        max_length=255, blank=True,
        help_text="The name of the model class you's like to analyse."
    )
    label = models.CharField(
        max_length=255, blank=True, help_text="A label of the chart."
    )
    field_path = models.CharField(
        max_length=255, blank=True, help_text="The constructor of to the plotted value."
    )
    chart_types = models.ManyToManyField(
        ChartType, blank=True, help_text="A selection of chart types which should be accessible."
    )
    help_text = models.CharField(
        max_length=255, blank=True, help_text="Contains a description of the chart"
    )
    legend_x = models.CharField(
        max_length=255, blank=True, help_text="Text for the legend of the x-axis"
    )
    legend_y = models.CharField(
        max_length=255, blank=True, help_text="Text for the legend of the y-axis"
    )

    def __str__(self):
        return "{} ({})".format(self.label, self.field_path)



import json

from django.core.exceptions import ObjectDoesNotExist, FieldError
from django.contrib.contenttypes.models import ContentType
from django.db.models import Count
from django.views.generic import TemplateView
from django.views.generic.list import ListView

from . models import ChartConfig


class ChartSelector(ListView):
    model = ChartConfig
    template_name = 'charts/select_chart.html'


class DynChartView(TemplateView):

    template_name = 'charts/dynchart.html'

    def get_context_data(self, **kwargs):
        context = super(DynChartView, self).get_context_data()
        model_name = self.kwargs['model_name']
        try:
            ct = ContentType.objects.get(model=model_name).model_class()
        except ObjectDoesNotExist:
            context['fatal_error'] = True
            context['error_msg'] = "The model: <code>{}</code> you requested is not defined"\
                .format(model_name.title())
            return context

        property_name = self.kwargs['property']
        context['property_name'] = property_name
        try:
            chart = ChartConfig.objects.get(
                field_path=property_name,
                model_name=model_name
            )
            chart = chart.__dict__
        except ObjectDoesNotExist:
            context['error'] = True
            context['error_msg'] = """The ChartConfig Object:
                <code>field_path={}, model_name={}</code> you requested couldn't be found."""\
                .format(property_name, model_name.title())
            chart = {
                'legend_x': None,
                'legend_y': None,
                'label': 'not defined',
                'help_text': 'not defined'
            }

        context['charttype'] = self.kwargs['charttype']
        modelname = ct.__name__
        payload = []
        objects = ct.objects.all()
        try:
            for x in objects.values(property_name).annotate(
                    amount=Count(property_name)).order_by(property_name):
                if x[property_name]:
                    payload.append(["{}".format(x[property_name]), x['amount']])
                else:
                    payload.append(['None', x['amount']])
            context['all'] = objects.count()
            if chart['legend_x']:
                legendx = chart['legend_x']
            else:
                legendx = "# of {}s".format(modelname)
            data = {
                "items": "{} out of {}".format(objects.count(), context['all']),
                "title": "{}".format(chart['label']),
                "subtitle": "{}".format(chart["help_text"]),
                "legendy": chart["legend_y"],
                "legendx": legendx,
                "categories": "sorted(dates)",
                "measuredObject": "{}s".format(modelname),
                "ymin": 0,
                "payload": payload
            }
            context['data'] = data
        except FieldError:
            context['error'] = True
            context['error_msg'] = "The field: <code>{}</code> you requested does not exist"\
                .format(property_name)

        return context



# Generated by Django 2.0.2 on 2018-06-01 14:14

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ChartConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model_name', models.CharField(blank=True, help_text="The name of the model class you's like to analyse.", max_length=255)),
                ('label', models.CharField(blank=True, help_text='A label of the chart.', max_length=255)),
                ('field_path', models.CharField(blank=True, help_text='The constructor of to the plotted value.', max_length=255)),
                ('help_text', models.CharField(blank=True, help_text='Contains a description of the chart', max_length=255)),
                ('legend_x', models.CharField(blank=True, help_text='Text for the legend of the x-axis', max_length=255)),
                ('legend_y', models.CharField(blank=True, help_text='Text for the legend of the y-axis', max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='ChartType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, help_text='A name of the chart type.', max_length=255)),
                ('icon', models.CharField(blank=True, help_text='A HTML snippet which could be used to display a link to the chart', max_length=255)),
            ],
        ),
        migrations.AddField(
            model_name='chartconfig',
            name='chart_types',
            field=models.ManyToManyField(blank=True, help_text='A selection of chart types which should be accessible.', to='charts.ChartType'),
        ),
    ]



from django.conf.urls import url
from . import views

app_name = 'charts'

urlpatterns = [
    url(r'^chartselector/$', views.ChartSelector.as_view(), name='chart_selector'),
    url(
        r'^chart/(?P<model_name>[\w\-]+)/(?P<property>[\w\-]+)/(?P<charttype>[\w\-]+)/$',
        views.DynChartView.as_view(), name='dynchart'
    ),
]
